import os
import json
from django.test import TestCase
from edge.models import Genome, Operation, Fragment, Genome_Fragment
from edge.blastdb import build_all_genome_dbs, fragment_fasta_fn
from edge.crispr import find_crispr_target, crispr_dsb
from Bio.Seq import Seq


class GenomeCrisprDSBTest(TestCase):

    def build_genome(self, circular, *sequences):
        g = Genome(name='Foo')
        g.save()
        for seq in sequences:
            f = Fragment.create_with_sequence('Bar', seq, circular=circular)
            Genome_Fragment(genome=g, fragment=f, inherited=False).save()
            try:
                os.unlink(fragment_fasta_fn(f))
            except:
                pass
        build_all_genome_dbs(refresh=True)
        return Genome.objects.get(pk=g.id)

    def test_find_crispr_target_finds_target_on_forward_strand(self):
        s1 = 'agaaggtctggtagcgatgtagtcgatct'
        s2 = 'gactaggtacgtagtcgtcaggtcagtca'
        pam = 'cgg'
        g = self.build_genome(False, s1+pam+s2)
        guide = s1[-20:]
        t = find_crispr_target(g, guide, 'ngg')
        self.assertEquals(len(t), 1)
        self.assertEquals(t[0].fragment_id, g.fragments.all()[0].id)
        self.assertEquals(t[0].fragment_name, g.fragments.all()[0].name)
        self.assertEquals(t[0].subject_start, s1.index(guide)+1)
        self.assertEquals(t[0].subject_end, len(s1))
        self.assertEquals(t[0].pam, 'ngg')

    def test_find_crispr_target_only_finds_perfect_match_to_guide(self):
        s1 = 'agaaggtctggtagcgatgtagtcgatct'
        s2 = 'gactaggtacgtagtcgtcaggtcagtca'
        pam = 'cgg'
        g = self.build_genome(False, s1+pam+s2)
        guide = 'aaaaa'+s1[-15:]
        t = find_crispr_target(g, guide, 'ngg')
        self.assertEquals(len(t), 0)

    def test_find_crispr_target_finds_target_on_reverse_strand(self):
        s1 = 'agaaggtctggtagcgatgtagtcgatct'
        s2 = 'gactaggtacgtagtcgtcaggtcagtca'
        pam = 'cgg'
        g = self.build_genome(False, str(Seq(s1+pam+s2).reverse_complement()))
        guide = s1[-20:]
        t = find_crispr_target(g, guide, 'ngg')
        self.assertEquals(len(t), 1)
        self.assertEquals(t[0].fragment_id, g.fragments.all()[0].id)
        self.assertEquals(t[0].fragment_name, g.fragments.all()[0].name)
        self.assertEquals(t[0].subject_end, len(s2)+3+1)
        self.assertEquals(t[0].subject_start, len(s2)+3+1+20-1)
        self.assertEquals(t[0].pam, 'ngg')

    def test_find_crispr_target_finds_multiple_crispr_targets(self):
        s1 = 'agaaggtctggtagcgatgtagtcgatct'
        s2 = 'gactaggtacgtagtcgtcaggtcagtca'
        pam = 'cgg'
        g = self.build_genome(False, (s1+pam+s2)+str(Seq(s1+pam+s2).reverse_complement()))
        guide = s1[-20:]
        t = find_crispr_target(g, guide, 'ngg')
        self.assertEquals(len(t), 2)
        self.assertEquals(t[0].fragment_id, g.fragments.all()[0].id)
        self.assertEquals(t[0].fragment_name, g.fragments.all()[0].name)
        self.assertEquals(t[0].subject_start, s1.index(guide)+1)
        self.assertEquals(t[0].subject_end, len(s1))
        self.assertEquals(t[0].pam, 'ngg')
        self.assertEquals(t[1].fragment_id, g.fragments.all()[0].id)
        self.assertEquals(t[1].fragment_name, g.fragments.all()[0].name)
        self.assertEquals(t[1].subject_end, len(s1+pam+s2)+len(s2)+3+1)
        self.assertEquals(t[1].subject_start, len(s1+pam+s2)+len(s2)+3+1+20-1)
        self.assertEquals(t[1].pam, 'ngg')

    def test_find_crispr_target_does_not_find_target_without_pam(self):
        s1 = 'agaaggtctggtagcgatgtagtcgatct'
        s2 = 'gactaggtacgtagtcgtcaggtcagtca'
        pam = 'ccc'
        g = self.build_genome(False, s1+pam+s2)
        guide = s1[-20:]
        t = find_crispr_target(g, guide, 'ngg')
        self.assertEquals(len(t), 0)

    def test_find_crispr_target_does_not_find_target_with_part_of_pam(self):
        s1 = 'agaaggtctggtagcgatgtagtcgatct'
        s2 = 'gactaggtacgtagtcgtcaggtcagtca'
        pam = 'cgc'
        g = self.build_genome(False, s1+pam+s2)
        guide = s1[-20:]
        t = find_crispr_target(g, guide, 'ngg')
        self.assertEquals(len(t), 0)

    def test_find_crispr_target_finds_target_across_circular_boundary(self):
        s1 = 'agaaggtctggtagcgatgtagtcgatct'
        s2 = 'gactaggtacgtagtcgtcaggtcagtca'
        pam = 'cgg'
        s = s1[10:]+pam+s2+s1[0:10]
        g = self.build_genome(True, s)
        guide = s1[-20:]
        t = find_crispr_target(g, guide, 'ngg')
        self.assertEquals(len(t), 1)
        self.assertEquals(t[0].fragment_id, g.fragments.all()[0].id)
        self.assertEquals(t[0].fragment_name, g.fragments.all()[0].name)
        self.assertEquals(t[0].subject_start, (s1.index(guide)+1-10-1) % len(s)+1)
        self.assertEquals(t[0].subject_end, (len(s1)-10-1) % len(s)+1)
        self.assertEquals(t[0].pam, 'ngg')

    def test_find_crispr_target_finds_target_with_pam_across_circular_boundary(self):
        s1 = 'agaaggtctggtagcgatgtagtcgatct'
        s2 = 'gactaggtacgtagtcgtcaggtcagtca'
        pam = 'cgg'
        s = pam[1:]+s2+s1+pam[:1]
        g = self.build_genome(True, s)
        guide = s1[-20:]
        t = find_crispr_target(g, guide, 'ngg')
        self.assertEquals(len(t), 1)
        self.assertEquals(t[0].fragment_id, g.fragments.all()[0].id)
        self.assertEquals(t[0].fragment_name, g.fragments.all()[0].name)
        self.assertEquals(t[0].subject_start, len(pam[1:]+s2)+s1.index(guide)+1)
        self.assertEquals(t[0].subject_end, len(s)-1)
        self.assertEquals(t[0].pam, 'ngg')

    def test_find_crispr_target_finds_reverse_complement_across_circular_boundary(self):
        s1 = 'agaaggtctggtagcgatgtagtcgatct'
        s2 = 'gactaggtacgtagtcgtcaggtcagtca'
        pam = 'cgg'
        s = s1[10:]+pam+s2+s1[0:10]
        g = self.build_genome(True, str(Seq(s).reverse_complement()))
        guide = s1[-20:]
        t = find_crispr_target(g, guide, 'ngg')
        self.assertEquals(len(t), 1)
        self.assertEquals(t[0].fragment_id, g.fragments.all()[0].id)
        self.assertEquals(t[0].fragment_name, g.fragments.all()[0].name)
        self.assertEquals(t[0].subject_start, (len(s)-(s1.index(guide)+1-10)) % len(s)+1)
        self.assertEquals(t[0].subject_end, (len(s)-(len(s1)-10)) % len(s)+1)
        self.assertEquals(t[0].pam, 'ngg')

    def test_find_crispr_target_finds_reverse_complement_with_pam_across_circular_boundary(self):
        s1 = 'agaaggtctggtagcgatgtagtcgatct'
        s2 = 'gactaggtacgtagtcgtcaggtcagtca'
        pam = 'cgg'
        s = pam[1:]+s2+s1+pam[:1]
        g = self.build_genome(True, str(Seq(s).reverse_complement()))
        guide = s1[-20:]
        t = find_crispr_target(g, guide, 'ngg')
        self.assertEquals(len(t), 1)
        self.assertEquals(t[0].fragment_id, g.fragments.all()[0].id)
        self.assertEquals(t[0].fragment_name, g.fragments.all()[0].name)
        self.assertEquals(t[0].subject_end, 2)
        self.assertEquals(t[0].subject_start, 2+len(guide)-1)
        self.assertEquals(t[0].pam, 'ngg')

    def test_crispr_dsb_finds_and_annotates_target_on_forward_strand(self):
        s1 = 'agaaggtctggtagcgatgtagtcgatct'
        s2 = 'gactaggtacgtagtcgtcaggtcagtca'
        pam = 'cgg'
        g = self.build_genome(False, s1+pam+s2)
        guide = s1[-20:]

        a = g.fragments.all()[0].indexed_fragment().annotations()
        self.assertEquals(len(a), 0)

        c = crispr_dsb(g, guide, 'ngg')
        self.assertNotEquals(c.id, g.id)
        self.assertEquals(c.parent.id, g.id)

        a = c.fragments.all()[0].indexed_fragment().annotations()
        self.assertEquals(len(a), 1)
        self.assertEquals(a[0].base_first, s1.index(guide)+1)
        self.assertEquals(a[0].base_last, len(s1))
        self.assertEquals(a[0].feature_base_first, 1)
        self.assertEquals(a[0].feature_base_last, len(guide))
        self.assertEquals(a[0].feature.strand, 1)
        self.assertEquals(a[0].feature.name, 'CRISPR-Cas9 (pam ngg) target')
        self.assertEquals(a[0].feature.operation.type, Operation.CRISPR_DSB[0])
        self.assertEquals(a[0].feature.operation.genome, c)

        # annotation is visible in parent genome, since it's not on new base pairs
        a = g.fragments.all()[0].indexed_fragment().annotations()
        self.assertEquals(len(a), 1)
        self.assertEquals(a[0].feature.operation.genome, c)

    def test_crispr_dsb_finds_and_annotates_target_on_reverse_strand(self):
        s1 = 'agaaggtctggtagcgatgtagtcgatct'
        s2 = 'gactaggtacgtagtcgtcaggtcagtca'
        pam = 'cgg'
        g = self.build_genome(False, str(Seq(s1+pam+s2).reverse_complement()))
        guide = s1[-20:]

        c = crispr_dsb(g, guide, 'ngg')
        self.assertNotEquals(c.id, g.id)
        self.assertEquals(c.parent.id, g.id)

        a = c.fragments.all()[0].indexed_fragment().annotations()
        self.assertEquals(len(a), 1)
        self.assertEquals(a[0].base_first, len(s2)+3+1)
        self.assertEquals(a[0].base_last, len(s2)+3+1+20-1)
        self.assertEquals(a[0].feature_base_first, 1)
        self.assertEquals(a[0].feature_base_last, len(guide))
        self.assertEquals(a[0].feature.strand, -1)
        self.assertEquals(a[0].feature.name, 'CRISPR-Cas9 (pam ngg) target')
        self.assertEquals(a[0].feature.operation.type, Operation.CRISPR_DSB[0])
        self.assertEquals(a[0].feature.operation.genome, c)

    def test_crispr_dsb_finds_and_annotates_target_across_circular_boundary(self):
        s1 = 'agaaggtctggtagcgatgtagtcgatct'
        s2 = 'gactaggtacgtagtcgtcaggtcagtca'
        pam = 'cgg'
        s = s1[-15:]+pam+s2+s1[0:len(s1)-15]
        g = self.build_genome(True, s)
        guide = s1[-20:]

        c = crispr_dsb(g, guide, 'ngg')
        self.assertNotEquals(c.id, g.id)
        self.assertEquals(c.parent.id, g.id)

        a = c.fragments.all()[0].indexed_fragment().annotations()
        self.assertEquals(len(a), 2)
        self.assertEquals(a[0].base_first, 1)
        self.assertEquals(a[0].base_last, 15)
        self.assertEquals(a[0].feature_base_first, 6)
        self.assertEquals(a[0].feature_base_last, 20)
        self.assertEquals(a[0].feature.strand, 1)
        self.assertEquals(a[1].base_first, len(s)-5+1)
        self.assertEquals(a[1].base_last, len(s))
        self.assertEquals(a[1].feature_base_first, 1)
        self.assertEquals(a[1].feature_base_last, 5)
        self.assertEquals(a[1].feature.strand, 1)

    def test_crispr_dsb_finds_and_annotates_reverse_complement_across_circular_boundary(self):
        s1 = 'agaaggtctggtagcgatgtagtcgatct'
        s2 = 'gactaggtacgtagtcgtcaggtcagtca'
        pam = 'cgg'
        s = s1[-15:]+pam+s2+s1[0:len(s1)-15]
        g = self.build_genome(True, str(Seq(s).reverse_complement()))
        guide = s1[-20:]

        c = crispr_dsb(g, guide, 'ngg')
        self.assertNotEquals(c.id, g.id)
        self.assertEquals(c.parent.id, g.id)

        a = c.fragments.all()[0].indexed_fragment().annotations()
        self.assertEquals(len(a), 2)
        self.assertEquals(a[0].base_first, 1)
        self.assertEquals(a[0].base_last, 5)
        self.assertEquals(a[0].feature_base_first, 16)
        self.assertEquals(a[0].feature_base_last, 20)
        self.assertEquals(a[0].feature.strand, -1)

        self.assertEquals(a[1].base_first, len(s)-15+1)
        self.assertEquals(a[1].base_last, len(s))
        self.assertEquals(a[1].feature_base_first, 1)
        self.assertEquals(a[1].feature_base_last, 15)
        self.assertEquals(a[1].feature.strand, -1)

    def test_crispr_dsb_creates_operations(self):
        s1 = 'agaaggtctggtagcgatgtagtcgatct'
        s2 = 'gactaggtacgtagtcgtcaggtcagtca'
        pam = 'cgg'
        g = self.build_genome(False, s1+pam+s2)
        guide = s1[-20:]

        c = crispr_dsb(g, guide, 'ngg')
        self.assertEquals(g.operation_set.count(), 0)
        self.assertEquals(c.operation_set.count(), 1)
        self.assertEquals(c.operation_set.all()[0].type, Operation.CRISPR_DSB[0])

    def test_crispr_dsb_creates_new_fragment(self):
        s1 = 'agaaggtctggtagcgatgtagtcgatct'
        s2 = 'gactaggtacgtagtcgtcaggtcagtca'
        pam = 'cgg'
        g = self.build_genome(False, s1+pam+s2)
        guide = s1[-20:]
        c = crispr_dsb(g, guide, 'ngg')
        self.assertEquals(c.fragments.all()[0].parent, g.fragments.all()[0])

    def test_crispr_dsb_api_works(self):
        s1 = 'agaaggtctggtagcgatgtagtcgatct'
        s2 = 'gactaggtacgtagtcgtcaggtcagtca'
        pam = 'cgg'
        g = self.build_genome(False, s1+pam+s2)
        guide = s1[-20:]

        data = dict(genome_name='FooBar', notes='blah', guide=guide, pam='ngg', create=True)
        res = self.client.post('/edge/genomes/'+str(g.id)+'/crispr/dsb/', data=json.dumps(data),
                               content_type='application/json')
        self.assertEquals(res.status_code, 201)

        self.assertEquals(Operation.objects.count(), 1)
        self.assertEquals(g.children.count(), 1)
        c = g.children.all()[0]
        a = c.fragments.all()[0].indexed_fragment().annotations()
        self.assertEquals(len(a), 1)
        self.assertEquals(a[0].feature.operation.genome, c)
        self.assertEquals(c.operation_set.all()[0].type, Operation.CRISPR_DSB[0])
        self.assertEquals(c.operation_set.all()[0].params, json.dumps(dict(guide=guide, pam='ngg')))

    def test_multiple_api_calls_return_same_child(self):
        s1 = 'agaaggtctggtagcgatgtagtcgatct'
        s2 = 'gactaggtacgtagtcgtcaggtcagtca'
        pam = 'cgg'
        g = self.build_genome(False, s1+pam+s2)
        guide = s1[-20:]
        data = dict(genome_name='FooBar', notes='blah', guide=guide, pam='ngg', create=True)

        res = self.client.post('/edge/genomes/'+str(g.id)+'/crispr/dsb/', data=json.dumps(data),
                               content_type='application/json')
        self.assertEquals(res.status_code, 201)
        r = json.loads(res.content)
        c1 = r['id']

        res = self.client.post('/edge/genomes/'+str(g.id)+'/crispr/dsb/', data=json.dumps(data),
                               content_type='application/json')
        # returns 200 not 201
        self.assertEquals(res.status_code, 200)
        r = json.loads(res.content)
        c2 = r['id']
        self.assertEquals(c1, c2)



from django_assets import Bundle, register

js = Bundle('edge/src/js/*.js', filters='jsmin', output='edge/static/edge/edge.min.js')
register('edge_js', js)

js = Bundle('edge/src/js/*.js', output='edge/static/edge/edge.js')
register('edge', js)

css = Bundle('edge/src/css/app.css', output='edge/static/edge/edge.css')
register('edge_css', css)

jst = Bundle('edge/src/partials/*.html', filters='jst', output='edge/static/edge/edge_jst.js')
register('edge_jst', jst)



import re
import os
import tempfile
import subprocess
from django.conf import settings
from Bio.Seq import Seq


PRIMER3_MIN_PRIMER = 18
PRIMER3_MAX_PRIMER = 25

PRIMER3_INT_DEFAULTS = {
    'PRIMER_OPT_SIZE': 20,
    'PRIMER_PICK_INTERNAL_OLIGO': 1,
    'PRIMER_INTERNAL_MAX_SELF_END': 8,
    'PRIMER_MIN_SIZE': PRIMER3_MIN_PRIMER,
    'PRIMER_MAX_SIZE': PRIMER3_MAX_PRIMER,
    'PRIMER_MAX_NS_ACCEPTED': 0,
    'PRIMER_MAX_SELF_ANY': 12,
    'PRIMER_MAX_SELF_END': 8,
    'PRIMER_PAIR_MAX_COMPL_ANY': 12,
    'PRIMER_PAIR_MAX_COMPL_END': 8,
    'PRIMER_MAX_POLY_X': 100,
    'PRIMER_INTERNAL_MAX_POLY_X': 100,
}

PRIMER3_FLOAT_DEFAULTS = {
    'PRIMER_OPT_TM': 60.0,
    'PRIMER_MIN_TM': 57.0,
    'PRIMER_MAX_TM': 63.0,
    'PRIMER_MIN_GC': 20.0,
    'PRIMER_MAX_GC': 80.0,
    'PRIMER_SALT_MONOVALENT': 50.0,
    'PRIMER_DNA_CONC': 5000.0,  # nM
}


def parse_primer3_output(d):
    primers = {}

    for k in d:
        m = re.search('\d+', k)
        if m:
            i = int(m.group(0))
            if i not in primers:
                primers[i] = {}
            k2 = re.sub('_%s_' % i, '_', k)
            primers[i][k2] = d[k]

    return [primers[k] for k in sorted(primers.keys())]


def primer3_run(opts):
    fn = None
    with tempfile.NamedTemporaryFile(mode='w', delete=False) as f:
        fn = f.name
        for k in opts:
            f.write('%s=%s\n' % (k, opts[k]))
        f.write('PRIMER_THERMODYNAMIC_PARAMETERS_PATH=%s/primer3_config/\n' % settings.PRIMER3_DIR)
        f.write('=')

    cmd = "%s/primer3_core %s" % (settings.PRIMER3_DIR, fn)
    out = subprocess.check_output(cmd.split(' '))
    os.unlink(fn)
    r = {}
    for l in out.split('\n'):
        if '=' in l:
            l = l.split('=')
            r[l[0]] = '='.join(l[1:])

    return parse_primer3_output(r)


def design_primers_from_template(template, roi_start, roi_len, junctions, primer3_opts):
    primer3_opts = {} if primer3_opts is None else primer3_opts
    min_primer_product_size = min(PRIMER3_MIN_PRIMER*2+roi_len, len(template))
    max_primer_product_size = len(template)

    opts = {}
    opts.update(PRIMER3_INT_DEFAULTS)
    opts.update(PRIMER3_FLOAT_DEFAULTS)
    for k, v in primer3_opts.iteritems():
        if k in PRIMER3_INT_DEFAULTS:
            opts[k] = int(v)
        elif k in PRIMER3_FLOAT_DEFAULTS:
            opts[k] = float(v)

    opts.update(dict(PRIMER_PRODUCT_SIZE_RANGE='%s-%s' % (min_primer_product_size,
                                                          max_primer_product_size),
                     SEQUENCE_ID='_',
                     SEQUENCE_TEMPLATE=template,
                     SEQUENCE_TARGET='%s,%s' % (roi_start, roi_len)))

    primers = primer3_run(opts)
    if junctions and len(junctions) > 0:
        for primer in primers:
            p1 = primer['PRIMER_LEFT_SEQUENCE']
            p2 = primer['PRIMER_RIGHT_SEQUENCE']
            p2_rc = str(Seq(p2).reverse_complement())
            p1_i = template.lower().index(p1.lower())
            p2_i = template.lower().index(p2_rc.lower())
            primer['PRIMER_LEFT_SEQUENCE_DISTANCE_TO_JUNCTION'] = junctions[0]-(p1_i+len(p1))
            primer['PRIMER_RIGHT_SEQUENCE_DISTANCE_TO_JUNCTION'] = p2_i-junctions[-1]

    return primers


def design_primers(fragment, roi_start_bp, roi_len, upstream_window, downstream_window, opts):
    """
    Design primer using primer3 pipeline. Returns list of designed primers.
    """

    opts = {} if opts is None else opts
    fragment = fragment.indexed_fragment()

    roi = fragment.get_sequence(bp_lo=roi_start_bp, bp_hi=roi_start_bp+roi_len-1)
    bp_lo = roi_start_bp-upstream_window
    bp_hi = roi_start_bp+roi_len-1+downstream_window
    template = fragment.get_sequence(bp_lo=bp_lo, bp_hi=bp_hi)

    roi_start = template.index(roi)+1
    return design_primers_from_template(template, roi_start, len(roi),
                                        [upstream_window, upstream_window+len(roi)-1], opts)



#!/usr/bin/env python
from __future__ import print_function

import os
from subprocess import check_output, STDOUT

from setuptools import setup, find_packages, Command
from setuptools.command.sdist import sdist


class BowerInstallCommand(Command):
  description = 'run bower commands [install by default]'
  user_options = [
    ('bower-command=', 'c', 'Bower command to run. Defaults to "install".'),
    ('force-latest', 'F', 'Force latest version on conflict.'),
    ('allow-root', 'R', 'Allow bower to be run as root.'),
    ('production', 'p', 'Do not install project devDependencies.'),
  ]
  boolean_options = ['production', 'force-latest', 'allow-root']

  def initialize_options(self):
    self.force_latest = False
    self.production = False
    self.bower_command = 'install'
    self.allow_root = False

  def finalize_options(self):
    pass

  def run(self):
    cmd = ['bower', self.bower_command, '--no-color']
    if self.force_latest:
      cmd.append('-F')
    if self.production:
      cmd.append('-p')
    if self.allow_root:
      cmd.append('--allow-root')
    print(check_output(cmd, stderr=STDOUT))


class BuildAssetsCommand(Command):
  description = 'build django assets'
  user_options = []

  def initialize_options(self):
    pass

  def finalize_options(self):
    pass

  def run(self):
    src_dir = os.path.join(os.path.dirname(__file__), 'src')
    print(check_output(['python', 'manage.py', 'assets', 'build'], cwd=src_dir, stderr=STDOUT))


class SdistCommandWithJS(sdist):
  def run(self):
    self.run_command('install_bower')
    self.run_command('build_assets')
    sdist.run(self)


setup(
  name='edge-genome',
  version='0.3.0',

  author='Ginkgo Bioworks',
  author_email='devs@ginkgobioworks.com',
  url='https://github.com/ginkgobioworks/edge/',

  description='Genome Engineering Tool',
  long_description=open('README.rst').read(),

  license='MIT',
  keywords='edge genome annotate track query django database ginkgo',
  classifiers=[
    'Development Status :: 5 - Production/Stable',
    'Environment :: Web Environment',
    'Environment :: Other Environment',
    'Framework :: Django :: 1.6',
    'Intended Audience :: Developers',
    'Intended Audience :: Science/Research',
    'License :: OSI Approved :: MIT License',
    'Natural Language :: English',
    'Operating System :: OS Independent',
    'Programming Language :: Python :: 2.7',
    'Programming Language :: JavaScript',
    'Programming Language :: SQL',
    'Topic :: Database',
    'Topic :: Internet :: WWW/HTTP :: Indexing/Search',
    'Topic :: Internet :: WWW/HTTP :: WSGI :: Application',
    'Topic :: Scientific/Engineering :: Artificial Intelligence',
    'Topic :: Scientific/Engineering :: Artificial Life',
    'Topic :: Scientific/Engineering :: Information Analysis',
    'Topic :: Software Development :: Libraries',
    'Topic :: Software Development :: Libraries :: Python Modules',
    'Topic :: Text Processing :: General',
  ],

  cmdclass={
    'install_bower': BowerInstallCommand,
    'build_assets': BuildAssetsCommand,
    'sdist': SdistCommandWithJS,
  },

  package_dir={'': 'src'},
  packages=find_packages('src', exclude=['*.migrations', '*server*']),
  include_package_data=True,
  zip_safe=True,

  setup_requires=[
    'django_assets',
  ],
  install_requires=[
    'django < 1.7',
    'jsmin',
    'celery < 4.0',
    'django-celery',
    'bcbio-gff',
    'pytz >= 1023b',
  ],
  tests_require=[
    'nose'
    'django-nose < 1.4',
    'flake8',
    'mysql-python',
  ],
)
